@@ -3,7 +3,7 @@ use Carp;
 use strict;
 use Class::ISA;
 
-$CGI::Application::VERSION = '4.19_1';
+$CGI::Application::VERSION = '4.32_1';
 
 my %INSTALLED_CALLBACKS = (
 #	hook name          package                 sub
@@ -21,6 +21,7 @@ my %INSTALLED_CALLBACKS = (
 
 sub new {
 	my $class = shift;
+
 	my @args = @_;
 
 	if (ref($class)) {
@@ -195,21 +196,32 @@ sub run {
 	# Call cgiapp_postrun() hook
 	$self->call_hook('postrun', \$body);
 
-	# Set up HTTP headers
-	my $headers = $self->_send_headers();
+    my $return_value;
+    if ($self->{__IS_PSGI}) {
+        my ($status, $headers) = $self->_send_psgi_headers();
+        $return_value = [ $status, $headers, [ $body ]];
+    }
+    else {
+        # Set up HTTP headers non-PSGI responses
+        my $headers = $self->_send_headers();
 
-	# Build up total output
-	my $output  = $headers.$body;
-
-	# Send output to browser (unless we're in serious debug mode!)
-	unless ($ENV{CGI_APP_RETURN_ONLY}) {
-		print $output;
-	}
+        # Build up total output
+        $return_value  = $headers.$body;
+        print $return_value unless $ENV{CGI_APP_RETURN_ONLY};
+    }
 
 	# clean up operations
 	$self->call_hook('teardown');
 
-	return $output;
+	return $return_value;
+}
+
+sub run_as_psgi {
+    my $self = shift;
+    $self->{__IS_PSGI} = 1;
+
+    # Run doesn't officially support any args, but pass them through in case some sub-class uses them.
+    return $self->run(@_);
 }
 
 
@@ -464,18 +476,14 @@ sub query {
 	my $self = shift;
 	my ($query) = @_;
 
-	# We're only allowed to set a new query object if one does not yet exist!
-	unless (exists($self->{__QUERY_OBJ})) {
-		my $new_query_obj;
-
-		# If data is provided, set it!  Otherwise, create a new one.
-		if (defined($query)) {
-			$new_query_obj = $query;
-		} else {
-			$new_query_obj = $self->cgiapp_get_query();
+	# If data is provided, set it!  Otherwise, create a new one.
+	if (defined($query)) {
+		$self->{__QUERY_OBJ} = $query;
+	} else {
+		# We're only allowed to create a new query object if one does not yet exist!
+		unless (exists($self->{__QUERY_OBJ})) {
+			$self->{__QUERY_OBJ} = $self->cgiapp_get_query();
 		}
-
-		$self->{__QUERY_OBJ} = $new_query_obj;
 	}
 
 	return $self->{__QUERY_OBJ};
@@ -604,6 +612,7 @@ sub get_current_runmode {
 ###########################
 
 
+# return headers as a string
 sub _send_headers {
 	my $self = shift;
 	my $q    = $self->query;
@@ -616,6 +625,20 @@ sub _send_headers {
       : croak "Invalid header_type '$type'"
 }
 
+# return a 2 element array modeling the first PSGI redirect values: status code and arrayref of header pairs
+sub _send_psgi_headers {
+	my $self = shift;
+	my $q    = $self->query;
+	my $type = $self->header_type;
+
+    return
+        $type eq 'redirect' ? $q->psgi_redirect( $self->header_props )
+      : $type eq 'header'   ? $q->psgi_header  ( $self->header_props )
+      : $type eq 'none'     ? ''
+      : croak "Invalid header_type '$type'"
+
+}
+
 
 # Make all hash keys CAPITAL
 # although this method is internal, some other extensions
@@ -974,7 +997,50 @@ data returned is print()'ed to STDOUT and to the browser.  If
 the specified mode is not found in the run_modes() table, run() will
 croak().
 
-=head2 Methods to possibly override 
+=head3 run_as_psgi
+
+ my $psgi_aref = $webapp->run_as_psgi;
+
+Just like C<< run >>, but prints no output and returns the data structure
+required by the L<PSGI> specification. Use this if you want to run the
+application on top of a PSGI-compatible handler, such as L<Plack> provides.
+
+If you are just getting started, just use C<< run() >>. It's easy to switch to using
+C<< run_as_psgi >> later.
+
+Why use C<< run_as_psgi() >>? There are already solutions to run
+CGI::Application-based projects on several web servers with dozens of plugins.
+Running as a PSGI-compatible application provides the ability to run on
+additional PSGI-compatible servers, as well as providing access to all of the
+"Middleware" solutions available through the L<Plack> project.
+
+The structure returned is an arrayref, containing the status code, an arrayref
+of header key/values and an arrayref containing the body.
+
+ [ 200, [ 'Content-Type' => 'text/html' ], [ $body ] ]
+
+By default the body is a single scalar, but plugins may modify this to return
+other value PSGI values.  See L<PSGI/"The Response"> for details about the
+response format.
+
+Note that calling C<< run_as_psgi >> only handles the I<output> portion of the
+PSGI spec. to handle the input, you need to use a CGI.pm-like query object that
+is PSGI-compliant, such as L<CGI::PSGI>. This query object must provide L<psgi_header>
+and L<psgi_redirect> methods.
+
+The final result might look like this:
+
+    use WebApp;
+    use CGI::PSGI;
+
+    my $handler = sub {
+        my $env = shift;
+        my $webapp = WebApp->new({ QUERY => CGI::PSGI->new($env) });
+        $webapp->run_as_psgi;
+    };
+
+
+=head2 Methods to possibly override
 
 CGI::Application implements some methods which are expected to be overridden
 by implementing them in your sub-class module.  These methods are as follows:
@@ -1037,7 +1103,7 @@ CGI::Application.  The cgiapp_init() method receives, as its parameters,
 all the arguments which were sent to the new() method.
 
 An example of the benefits provided by utilizing this hook is
-creating a custom "application super-class" from which which all
+creating a custom "application super-class" from which all
 your web applications would inherit, instead of CGI::Application.
 
 Consider the following:
@@ -1165,13 +1231,13 @@ modes, and when a C<param()> is a particular value.
 Override this method to retrieve the query object if you wish to use a
 different query interface instead of CGI.pm.  
 
-CGI.pm is only loaded to provided query object is only loaded if it used on a given request.
+CGI.pm is only loaded if it is used on a given request.
 
 If you can use an alternative to CGI.pm, it needs to have some compatibility
 with the CGI.pm API. For normal use, just having a compatible C<param> method
 should be sufficient. 
 
-If use the C<path_info> option to the mode_param() method, then we will call
+If you use the C<path_info> option to the mode_param() method, then we will call
 the C<path_info()> method on the query object.
 
 If you use the C<Dump> method in CGI::Application, we will call the C<Dump> and
@@ -1225,7 +1291,7 @@ This will allow the HTML::Template object to be further customized:
 Note that if you want to pass extra arguments but use the default template
 name, you still need to provide a name of C<undef>:
 
-    my $tmpl_obj = $webapp->load_tmpl(undef',
+    my $tmpl_obj = $webapp->load_tmpl(undef,
          die_on_bad_params => 0,
          cache => 1
     );
@@ -1246,9 +1312,12 @@ not a file. It features a simple syntax and MIME-type detection.
 
 B<specifying the template class with html_tmpl_class()>
 
-You may specify an API-compatible alternative to HTML::Template by overriding
-C<html_tmpl_class()>. This method should return the class name of your template
-system. The default simply returns "HTML::Template". The alternate class should
+You may specify an API-compatible alternative to L<HTML::Template> by setting
+a new C<html_tmpl_class()>:
+
+  $self->html_tmpl_class('HTML::Template::Dumper');
+
+The default is "HTML::Template". The alternate class should
 provide at least the following parts of the HTML::Template API:
 
  $t = $class->new( scalarref => ... );  # If you use scalarref templates
@@ -1256,9 +1325,23 @@ provide at least the following parts of the HTML::Template API:
  $t = $class->new( filename => ... );
  $t->param(...); 
 
-Example implementation:
+Here's an example case allowing you to precisely test what's sent to your
+templates:
 
- sub html_tmpl_class { 'HTML::Template::Pro' }
+    $ENV{CGI_APP_RETURN_ONLY} = 1;
+    my $webapp = WebApp->new;
+       $webapp->html_tmpl_class('HTML::Template::Dumper'); 
+    my $out_str = $webapp->run;
+    my $tmpl_href = eval "$out_str";
+
+    # Now Precisely test what would be set to the template
+    is ($tmpl_href->{pet_name}, 'Daisy', "Daisy is sent template");
+
+This is a powerful technique because HTML::Template::Dumper loads and considers
+the template file that would actually be used. If the 'pet_name' token was missing
+in the template, the above test would fail. So, you are testing both your code
+and your templates in a much more precise way than using simple regular
+expressions to see if the string "Daisy" appeared somewhere on the page.
 
 B<The load_tmpl() callback>
 
@@ -1345,6 +1428,12 @@ If, for some reason, you want to use your own CGI query object, the new()
 method supports passing in your existing query object on construction using
 the QUERY attribute.
 
+There are a few rare situations where you want your own query object to be 
+used after your Application Module has already been constructed. In that case 
+you can pass it to c<query()> like this:
+
+    $webapp->query($new_query_object);
+    my $q = $webapp->query(); # now uses $new_query_object
 
 =head3 run_modes()
 
@@ -1638,7 +1727,19 @@ for a cron script!
 
 =cut
 
-sub html_tmpl_class { 'HTML::Template' }
+sub html_tmpl_class { 
+    my $self = shift;
+    my $tmpl_class = shift;
+
+	# First use?  Create new __ERROR_MODE
+	$self->{__HTML_TMPL_CLASS} = 'HTML::Template' unless (exists($self->{__HTML_TMPL_CLASS}));
+
+    if (defined $tmpl_class) {
+        $self->{__HTML_TMPL_CLASS} = $tmpl_class;
+    }
+
+    return $self->{__HTML_TMPL_CLASS};
+}
 
 sub load_tmpl {
 	my $self = shift;
@@ -1908,7 +2009,7 @@ meaning that a local web-browser would be used, allowing testing of websites
 that contain JavaScript.
 
 Direct testing is also easy. CGI::Application will normally print the output of it's
-run modes directly to STDOUT. This can be surprised with an enviroment variable, 
+run modes directly to STDOUT. This can be suppressed with an enviroment variable, 
 CGI_APP_RETURN_ONLY. For example:
 
   $ENV{CGI_APP_RETURN_ONLY} = 1;
@@ -1938,7 +2039,7 @@ L<CGI::Application::Plugin::ConfigAuto> - Keeping your config details in a separ
 
 =item *
 
-L<CGI::Application::Plugin::DBH> - Provides easy management of one or more database handles and candelay making the database connection until the moment it is actually used. 
+L<CGI::Application::Plugin::DBH> - Provides easy management of one or more database handles and can delay making the database connection until the moment it is actually used. 
 
 =item *
 
